@@ -23,6 +23,9 @@
 #' @param weight.sys a string. Weights may add to one ('SUM') or their product is equal to 1 ('PROD', default).
 #' @param theta a number. A parameter if \code{weight.sys='SUM'}, default is 2.  
 #' @param Wfix a logical parameter (default=FALSE). If TRUE the algorithm does not use adaptive distances.
+#' @param atleast integer. Check for degeneration of the map into a very low number of
+#' voronoi sets. (default 2) 2 means that the map will have at least 2 neurons 
+#' attracting data instances in their voronoi sets. 
 #' @param verbose a logical parameter (default=FALSE). If TRUE details of 
 #' computation are shown during the execution.
 #' #'         
@@ -69,9 +72,9 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
                                        niter=30,repetitions ,
                                        simplify=FALSE,
                                        qua=10,
-                                       standardize=FALSE, schema=4,
+                                       standardize=FALSE, schema=6,
                                        init.weights='EQUAL',weight.sys='PROD',
-                                       theta=2,Wfix=FALSE,verbose=FALSE){
+                                       theta=2,Wfix=FALSE,verbose=FALSE,atleast=2){
   tol=1e-10
   if(weight.sys=='PROD') {
     weight_sys=1
@@ -86,13 +89,24 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
   vars=ncol(x@M)
   ## we homogeneize data for speeding up the code if required
   tmp=Prepare(x,simplify,qua,standardize)
+  
   MM=tmp$MM
   x=tmp$x
+  remove(tmp)
   # end of the preprocessing step
   
   TOTSSQ=0
+  SSQ_comp=rep(0,vars*2)
+  tmpc=as.matrix(get.MatH.stats(x)$mat)
+  
+  colm=colMeans(tmpc)
   for (v in 1:vars){
-    TOTSSQ=TOTSSQ+(WH.SSQ(x[,v]))
+    tmpSSQ=WH.SSQ(x[,v])
+    TOTSSQ=TOTSSQ+tmpSSQ
+    C_comp=sum((tmpc[,v]-colm[v])^2)
+    V_comp=tmpSSQ-C_comp
+    SSQ_comp[v*2-1]=C_comp
+    SSQ_comp[v*2]=V_comp
   }
   ##------------------------------------------------batchKOHONENMAP-----------------------------------------------------
   solutions=list()
@@ -102,23 +116,21 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
   if (missing(repetitions)){
     repetitions=5
   }
-  #print(repetitions)
-  #proto=new("MatH",list(new("distributionH")),nrows=k,ncols=vars)
-  for (repet in (1:repetitions)){
-    
-    
-    #    data=x@M
-    #    nd=nrow(data)
+  
+  restart=1
+  repet=0
+  
+  
+  while(repet<repetitions){
+    repet=repet+1
     #random selection of prototypes of neurons
     if(k<=ind){
       proto=x[sample(1L:ind,k, replace=FALSE),]
     }else{
       proto=x[sample(1L:ind,k, replace=TRUE),]
     }
+    
     rownames(proto@M)=paste0("Prot. ",1:k)
-    # init=data[sample(1L:nd,k, replace=FALSE), ,drop=FALSE]
-    # proto=new("MatH")
-    # proto@M=init
     nhbrdist=as.matrix(dist(MAP$pts))
     dmax=(max(MAP$pts[,1])-min(MAP$pts[,1]))^2+(max(MAP$pts[,2])-min(MAP$pts[,2]))^2
     dmax=(max(as.matrix(dist(MAP$pts))))^2
@@ -129,17 +141,72 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
     if (Tmin<0){
       Tmin=sqrt(-(1/4)/(2*log(0.01)))
     }
+    
     TT=TMAX
     KT=exp(-(nhbrdist^2)/(2*TT^2))
+    KT[which(KT<1e-10)]=0
     #initialize weights
     if (Wfix){lambdas=matrix(1,2*vars,k)}else{
       if (init.weights=='EQUAL'){
         cat("Weights initialization === EQUAL  \n")
         if (weight.sys=='PROD'){
           lambdas=matrix(1,2*vars,k)}
-        else{lambdas=matrix(1/vars,2*vars,k)}
+        else{
+          if (schema==1||schema==2||schema==3||schema==4){
+            lambdas=matrix(1/vars,2*vars,k)}
+          if(schema==5||schema==6){
+            lambdas=matrix(1/(2*vars),2*vars,k)
+          }
+        }
       }
-      else{#Random initialization
+      if (init.weights=='PROP'){
+        if (weight.sys=='PROD'){
+          # 1=A weight for each variable (default) 
+          # 2=A weight for the average and the dispersion component of each variable
+          # 3=Same as 1 but a different set of weights for each cluster (indipprod=1)
+          # 4=Same as 2 but a different set of weights for each cluster (indipprod=1)
+          # 5=Same as 1 but a different set of weights for each cluster (allprod=1)
+          # 6=Same as 2 but a different set of weights for each cluster (allprod=1)            
+          if (schema==1||schema==3||schema==5){
+            tmpvals=SSQ_comp[c(1:v)*2-1]+SSQ_comp[c(1:v)*2]
+            mg=(prod(tmpvals))^(1/v)
+            tmpw=mg/tmpvals
+            elew=rep(tmpw,2,each=2)
+            lambdas=matrix(rep(elew,k),nrow = 2*vars,ncol = k,byrow = FALSE)
+          }
+          if (schema==2||4){
+            tmpvalsc= SSQ_comp[c(1:v)*2-1]
+            tmpvalsv= SSQ_comp[c(1:v)*2]
+            mgc=(prod(tmpvalsc))^(1/v)
+            mgv=(prod(tmpvalsv))^(1/v)
+            
+            tmpwc=mgc/tmpvalsc
+            tmpwv=mgv/tmpvalsv
+            elew=rep(0,2*v)
+            elew[c(1:v)*2-1]=tmpwc
+            elew[c(1:v)*2]=tmpwv
+            
+            lambdas=matrix(rep(elew,k),nrow = 2*vars,ncol = k,byrow = FALSE)
+          }
+          if (schema==6){
+            tmpvals=SSQ_comp
+            mg=(prod(tmpvals))^(1/(2*v))
+            tmpw=mg/tmpvals
+            lambdas=matrix(rep(tmpw,k),nrow = 2*vars,ncol = k,byrow = FALSE)
+          }
+          
+        }else{
+          if ((schema==1||schema==2||schema==3||schema==4)){
+            lambdas=matrix(1/v,nrow = 2*vars,ncol = k,byrow = FALSE)
+          }
+          if (schema==5||schema==6){
+            lambdas=matrix(2/(2*v),nrow = 2*vars,ncol = k,byrow = FALSE)
+          }
+          
+        }
+        
+      }
+      if ((init.weights!='EQUAL')&(init.weights!='PROP')) {#Random initialization
         cat("Weights initialization === RANDOM  \n")
         m1=matrix(runif((vars*k),0.01,0.99),vars,k)
         m2=matrix(runif((vars*k),0.01,0.99),vars,k)
@@ -177,75 +244,55 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
         lambdas[(c(1:vars)*2),]=m2
       }
     }
+    
+    
+    
     #assign objects to closest neuron
     #compute adaptive distances to prototypes
     distances=array(0,dim=c(vars,k,2))
     diINDtoPROT=array(0,dim=c(ind,vars,k,2))
     # compute distances
     
-    TTMMPP=c_ComputeFast_L2_SQ_WASS_DMAT(MM,proto)
-    MD=TTMMPP$Dist
-    #browser()
+    MD=(c_ComputeFast_L2_SQ_WASS_DMAT(MM,proto))$Dist
+    KT[KT<1e-50]=0
     Dind2Neu=MD%*%KT
     
     #initialize matrix of memberships
-    IDX=apply(Dind2Neu,1,which.min)
+    IDX=max.col(-Dind2Neu, ties.method="first")
     memb=matrix(0,ind,k)
     for (i in (1:ind)){
       memb[i,IDX[i]]=1
     }
     cat(paste("---------> repetitions  ",repet,"\n"))
     ## initialize clusters and prototypes
-    #proto=new("MatH",list(new("distributionH")),nrows=k,ncols=vars)
+    
     SSQ1=matrix(0,k,vars)
     GenCrit=Inf
     ##  compute initial criterion
-    
-    SSQ=0
-    for (cluster in 1:k){
-      for (variables in (1:vars)){
-        for (indiv in 1:ind){
-          if (memb[indiv,cluster]>0)
-            SSQ=SSQ+Dind2Neu[indiv,IDX[indiv]]
-        }
-      }
-    }
+    tmpIDX=Table2(IDX, k, FALSE)
+    SSQ=sum(memb*Dind2Neu)
     GenCrit=SSQ
     OK=1 
     maxit=100
     
     t=0
-    while (TT>=Tmin){
-      t=t+1
-      TT=TMAX*(Tmin/TMAX)^(t/(niter-1))
-      KT=exp(-(nhbrdist^2)/(2*TT^2))
-      KT[which(KT<1e-100)]=0
-      #computing prototypes
-      CardinalityOfNeuron=apply(memb,2,sum)
-      
-      for (cluster in 1:k){
-        kern=rep(0,ind)
-        for (individuals in 1:ind){
-          kern[individuals]=KT[IDX[individuals],cluster]
-        }
-        for (variables in 1:vars){
-          if(max(kern)<1e-200){
-            kern[which(kern<1e-200)]=1e-200
-          }
-          M_tmp=matrix(kern/sum(kern),nrow=nrow(MM[[variables]]),ncol=ind,byrow=TRUE)
-          M_tmp=M_tmp*MM[[variables]][,1:ind]
-          if(length(which(is.nan(M_tmp)))>0){browser()}
-          x_tmp=rowSums(M_tmp)
-          p_tmp=MM[[variables]][,(ind+1)]
-          proto@M[cluster,variables][[1]]@x=x_tmp
-          proto@M[cluster,variables][[1]]@p=p_tmp
-          M_S=M_STD_H(proto@M[cluster,variables][[1]])
-          proto@M[cluster,variables][[1]]@m=M_S[1]
-          proto@M[cluster,variables][[1]]@s=M_S[2]
-        }
+    ft=10
+    fts=0
+    while ((TT>Tmin)||(fts<=ft)){
+      if (t<(niter-1)){
+        
+        t=t+1
+        TT=TMAX*(Tmin/TMAX)^(t/(niter-1))
+        KT=exp(-(nhbrdist^2)/(2*TT^2))
+        KT[which(KT<1e-20)]=0
       }
-      
-      
+      if((t==(niter-1))&(fts<=ft)){
+        fts=fts+1
+      }
+      #computing prototypes
+      CardinalityOfNeuron=colSums(memb)
+      proto=c_PROTO_KOHONEN(proto,k,ind,MM,vars,KT, IDX)
+      #########################################################################################
       #first compute distances using kernel
       #  computing distances
       
@@ -259,31 +306,72 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
         distances[j,,2]=distances[j,,2]+colSums(tmpD$DET[[j]]$DV*GRKT)
       }
       
-      diINDtoPROT=array(0,dim=c(ind,vars,k,2))
-      diINDtoPROT2=array(0,dim=c(ind,vars,k,2))
+      tmp2=matrix(0,ind,k)
       if (Wfix==FALSE){#Weights computation
         # S2.2) Weights computation
         #NEW
+        if (t>0){oldlambda=lambdas}
+        
         lambdas=c_ADA_F_WHEIGHT(list(as.matrix(distances[,,1]),as.matrix(distances[,,2])),
                                 k, vars, ind, schema, weight_sys,  theta, 1)
+        if (t>0){
+          
+          for (cluster in 1:k){
+            if (weight.sys=="PROD"){
+              if ((schema>=1 & schema<=4)&
+                  ((abs(prod(lambdas[,cluster])-1))>0.0001)){
+                print(cat("ini ",prod(lambdas[,cluster])))
+                lambdas[,cluster]=oldlambda[,cluster]
+                print(cat("t",t, "CLU", cluster, "new",prod(lambdas[,cluster])))
+                
+              }
+              if ((schema==5||schema==6)&
+                  ((abs(prod(lambdas[,cluster])-1))>0.0001)){
+                print(cat("ini ",prod(lambdas[,cluster])))
+                lambdas[,cluster]=oldlambda[,cluster]
+                print(cat("t",t, "CLU", cluster, "new",prod(lambdas[,cluster])))
+              }
+            }
+            if (weight.sys=="SUM"){
+              if ((schema>=1 & schema<=4)&
+                  ((abs(sum(lambdas[,cluster])-2))>0.0001)){
+                print(cat("ini ",sum(lambdas[,cluster])))
+                lambdas[,cluster]=oldlambda[,cluster]
+                print(cat("t",t, "CLU", cluster, "new",sum(lambdas[,cluster])))
+                
+                # browser()
+              }
+              if ((schema==5||schema==6)&
+                  ((abs(sum(lambdas[,cluster])-1))>0.0001)){
+                print(cat("ini ",sum(lambdas[,cluster])))
+                lambdas[,cluster]=oldlambda[,cluster]
+                print(cat("t",t, "CLU", cluster, "new",sum(lambdas[,cluster])))
+                
+              }
+            } 
+            
+            
+          }
+        }
         wM=(lambdas[((1:vars)*2-1),])^theta
         wV=(lambdas[(1:vars)*2,])^theta
         
+        
         #assign data to neurons
         ####################
         if (schema==1|schema==3){
           ####################
           for (variables in (1:vars)){
             tmpM=matrix(wM[variables,],ind,k,byrow = TRUE)
-            diINDtoPROT[,variables,,1]=tmpD$DET[[variables]]$D*tmpM
+            tmp2=tmp2+tmpD$DET[[variables]]$D*tmpM
           }
           ###############
         }else{
           for (variables in (1:vars)){
             tmpM=matrix(wM[variables,],ind,k,byrow = TRUE)
-            diINDtoPROT[,variables,,1]=tmpD$DET[[variables]]$DM*tmpM
+            tmp2=tmp2+tmpD$DET[[variables]]$DM*tmpM
             tmpM=matrix(wV[variables,],ind,k,byrow = TRUE)
-            diINDtoPROT[,variables,,2]=tmpD$DET[[variables]]$DV*tmpM
+            tmp2=tmp2+tmpD$DET[[variables]]$DV*tmpM
           }
           
         }
@@ -293,59 +381,76 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
         ####################
         if (schema==1|schema==3){#one weight for one variable
           for (variables in (1:vars)){
-            tmpM=matrix(1,ind,k,byrow = TRUE)
-            diINDtoPROT[,variables,,1]=tmpD$DET[[variables]]$D
+            tmp2=tmp2+tmpD$DET[[variables]]$D
           }
         }else{
           for (variables in (1:vars)){
-            diINDtoPROT[,variables,,1]=tmpD$DET[[variables]]$DM
-            
-            diINDtoPROT[,variables,,2]=tmpD$DET[[variables]]$DV
+            tmp2=tmp2+tmpD$DET[[variables]]$DM
+            tmp2=tmp2+tmpD$DET[[variables]]$DV
           }
         }
       }
-  #    browser()
-      tmp=apply(diINDtoPROT,c(1,3),sum)
-      # tmp[which(is.na(tmp))]=1e100
-      Dind2Neu=tmp%*%KT#matrix(Inf,ind,k)
+      remove(tmpD)
+      Dind2Neu=tmp2%*%KT#matrix(Inf,ind,k)
       
       #recompute criterion
-      IDX=apply(Dind2Neu,1,which.min)
+      IDX=max.col(-Dind2Neu, ties.method="first")
+      old_NUM=tmpIDX
+      tmpIDX=Table2(IDX, k, FALSE)
+      if (t>5){
+        if (length(tmpIDX)<(length(old_NUM)-10)){
+          print(t)
+          print(old_NUM)
+          print(tmpIDX)
+          browser()
+        }
+        
+      }
       OldCrit=GenCrit
       GenCrit=0
       for (individual in 1:ind){
         GenCrit=GenCrit+sum(Dind2Neu[individual, IDX[individual]])
       }
-      
       if (verbose) {cat(paste(t,GenCrit, "\n", sep="---->"))}
       if (is.na(GenCrit)){
-        browser()
+        
         print('A GREAT PROBLEM HERE!')
       }
       if (verbose) {print(print_map(MAP,IDX))}
-      #print(TT)
-      #print(max(KT))
-      #print(min(KT[which(KT>1e-16)]))
     }
     #crisp assignment
     
-    cardinality=table(IDX)
-    #dimnames(cardinality)$IDX=paste("Cl",1:k,sep=".")
-    solutions=c(solutions,list(solution=list(MAP=MAP, IDX=IDX,cardinality=cardinality,proto=proto,
-                                             Crit=GenCrit,weights.comp=lambdas,Weight.sys=weight.sys)))
-    criteria=c(criteria,GenCrit)
-    #plot(proto,type="DENS")
+    cardinality=Table2(IDX, k, FALSE)
+    names(IDX)=get.MatH.rownames(x)
+    colnames(lambdas)=get.MatH.rownames(proto)
+    rownames(lambdas)=paste(c("M","D"),rep(get.MatH.varnames(x),each=2),sep=".")
+    
+    if (length(cardinality)>=atleast){
+      solutions=c(solutions,list(solution=list(MAP=MAP, IDX=IDX,cardinality=cardinality,proto=proto,
+                                               Crit=GenCrit,weights.comp=lambdas,Weight.sys=weight.sys)))
+      criteria=c(criteria,GenCrit)
+    }else{
+      
+      repet=repet-1
+    }
+    
   }
+  
   FM=solutions[[which.min(criteria)]]$MAP
   FIDX=solutions[[which.min(criteria)]]$IDX
- # if (verbose){print_map(FM,FIDX,gr=TRUE)}
   print(print_map(FM,FIDX,gr=TRUE))
-  #print(dmax)
-  #print(TMAX)
-  #print(Tmin)
-  return(best.solution=list(solution=solutions[[which.min(criteria)]],
-                            repetitions=which.min(criteria),
-                            quality=1-min(criteria)/TOTSSQ,dmax=dmax,TMAX=TMAX,Tmin=Tmin))
+  
+  best.solution=list(solution=solutions[[which.min(criteria)]],
+                     repetitions=which.min(criteria),
+                     quality=1-min(criteria)/TOTSSQ,dmax=dmax,TMAX=TMAX,Tmin=Tmin,DATA=x)
+  memb=matrix(0,ind,k)
+  memb[(best.solution$solution$IDX-1)*ind+c(1:ind)]=1
+  resTOTSQ=c_WH_ADPT_KMEANS_TOTALSSQ(x,memb,m=1,
+                                     lambdas=best.solution$solution$weights.comp,
+                                     proto=best.solution$solution$proto)
+  TTSQ2=sum(resTOTSQ$TSQ2_m+resTOTSQ$TSQ2_v)
+  best.solution$quality2=1-min(criteria)/TTSQ2
+  return(best.solution)
 }
 
 # Batch SOM of HD -----
@@ -364,6 +469,8 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
 #' @param qua if \code{simplify=TRUE} number of equally spaced quantiles for recodify the histograms (default=10)
 #' @param standardize A logic value (default is FALSE). If TRUE, histogram-valued data are standardized,  variable by variable, 
 #' using the Wassertein based standard deviation. Use if one wants to have variables with std equal to one.   
+#' @param verbose a logical parameter (default=FALSE). If TRUE details of 
+#' computation are shown during the execution.
 #' @return a list with the results of the Batch Kohonen map
 #'@slot solution A list.Returns the best solution among the \code{repetitions}etitions, i.e. 
 #' the one having the minimum sum of squares criterion.
@@ -393,48 +500,30 @@ WH_2d_Adaptive_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangu
 #' @importFrom class somgrid
 #' @examples
 #' \dontrun{
-#' results=WH_2d_Kohonen_maps(x = BLOOD,k = 2,
-#'                                    net=list(xdim=2,ydim=3,topo=c('rectangular')), 
-#'                                    repetitions = 2,simplify = TRUE,
-#'                                    qua = 10,standardize = TRUE)
+#' results=WH_2d_Kohonen_maps(x = BLOOD,
+#'                            net=list(xdim=2,ydim=3,topo=c('rectangular')), 
+#'                            repetitions = 2,simplify = TRUE,
+#'                            qua = 10,standardize = TRUE)
 #'                                    }
 #' @export
-WH_2d_Kohonen_maps =function (x,net=list(xdim=4,ydim=3,topo=c('rectangular')),
-                              kern.param=2, TMAX=2, Tmin=0.2, 
-                              niter=30,repetitions=5 ,      
-                              simplify=FALSE,
-                              qua=10,
-                              standardize=FALSE){
+WH_2d_Kohonen_maps=function (x,net=list(xdim=4,ydim=3,topo=c('rectangular')),
+          kern.param=2, TMAX=2, Tmin=0.2, 
+          niter=30,repetitions=5 ,      
+          simplify=FALSE,
+          qua=10,
+          standardize=FALSE, verbose=FALSE){
   SOL=WH_2d_Adaptive_Kohonen_maps(x,net, kern.param, TMAX, Tmin, 
-                                  niter,repetitions ,
-                                  simplify,
-                                  qua,
-                                  standardize, schema=1,
-                                  init.weights='EQUAL',weight.sys='PROD',theta=2,Wfix=TRUE)
+                                       niter,repetitions ,
+                                       simplify,
+                                       qua,
+                                       standardize, schema=1,
+                                       init.weights='EQUAL',
+                                       weight.sys='PROD',theta=2,
+                                       Wfix=TRUE,verbose=verbose)
   
   return(SOL)
 }
 
-WH.ADPT.KOHONEN.SSQ=function(x,memb,m,lambdas,proto){
-  vars=ncol(x@M)
-  ind=nrow(x@M)
-  k=ncol(memb)
-  lambdas[is.na(lambdas)]=0
-  SSQ=0
-  for (indiv in 1:ind){
-    for (cluster in 1:k){
-      for (variables in (1:vars)){
-        tmpD=WassSqDistH(x@M[indiv,variables][[1]],proto@M[cluster,variables][[1]],details=T)
-        tmpD_mean=as.numeric(tmpD[2])
-        tmpD_centered=as.numeric(tmpD[1]-tmpD[2])
-        SSQ=SSQ+((memb[indiv,cluster])^m)*(lambdas[(variables*2-1),cluster]*tmpD_mean+lambdas[(variables*2),cluster]*tmpD_centered)
-        
-      }
-    }
-  }
-  return(SSQ)
-}
-
 print_map=function(MAP,IDX,gr=FALSE){
   pp=table(IDX)
   print(pp)
